@@ -1,5 +1,5 @@
 /* face.c -- face module.
-   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -18,7 +18,7 @@
    You should have received a copy of the GNU Lesser General Public
    License along with the m17n library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   02111-1307, USA.  */
+   Boston, MA 02110-1301 USA.  */
 
 /***en
     @addtogroup m17nFace
@@ -416,7 +416,7 @@ mface__init ()
   MFaceBoxProp *box;
 
   M17N_OBJECT_ADD_ARRAY (face_table, "Face");
-  Mface = msymbol_as_managing_key ("face");
+  Mface = msymbol_as_managing_key (" face");
   msymbol_put_func (Mface, Mtext_prop_serializer,
 		    M17N_FUNC (serialize_face));
   msymbol_put_func (Mface, Mtext_prop_deserializer,
