@@ -18,7 +18,7 @@
    You should have received a copy of the GNU Lesser General Public
    License along with the m17n library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   02111-1307, USA.  */
+   Boston, MA 02110-1301 USA.  */
 
 /***en
     @addtogroup m17nFont
@@ -1362,9 +1362,11 @@ mfont__set_spec_from_plist (MFont *spec, MPlist *plist)
   for (i = 0; ! MPLIST_TAIL_P (plist); i++, plist = MPLIST_NEXT (plist))
     {
       if (! MPLIST_SYMBOL_P (plist))
-	MERROR (MERROR_FONT, Mnil);	
+	MERROR (MERROR_FONT, Mnil);
       spec_list[i] = MPLIST_SYMBOL (plist);
     }
+  if (i == 0)
+    MERROR (MERROR_FONT, Mnil);
   registry = spec_list[i - 1];
   if (i > 1 && registry != Mnil)
     {
