@@ -18,7 +18,7 @@
    You should have received a copy of the GNU Lesser General Public
    License along with the m17n library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   02111-1307, USA.  */
+   Boston, MA 02110-1301 USA.  */
 
 #ifndef _M17N_INTERNAL_H_
 #define _M17N_INTERNAL_H_
@@ -382,7 +382,8 @@ typedef struct
   do {									\
     if (object)								\
       {									\
-	if (((M17NObject *) (object))->ref_count_extended)		\
+	if (((M17NObject *) (object))->ref_count_extended		\
+	    || mdebug__flags[MDEBUG_FINI])				\
 	  {								\
 	    if (m17n_object_unref (object) == 0)			\
 	      (object) = NULL;						\
